//
//      MainWindow.cpp
//

#include "SimpleMailer.hpp"
#include "MainWindow.hpp"
#include "SendingWindow.hpp"
#include "ReceivedWindow.hpp"
#include "OptionWindow.hpp"

#define MARGIN 16

//
// Constructor definition.
// The constructor of the parent class is called for various initializations.
//
MainWindow::MainWindow(Void) : SFRTitleWindow(SFRApplication::GetInstance(),
    SFXGraphics::GetDeviceRectangle(), "Simple Mailer") static_throws
{
    // Key handler registration
    if (static_try()) {
        static_throw(RegisterHandler(SFEVT_KEY, HANDLER_AFTER, HANDLER_FUNCTION(OnKey)));
    }

    // Get the font height
    SInt16 fontHeight = SFXGraphics::GetFontHeight(AEE_FONT_NORMAL);

    // Button width calculation
    SInt16 width = SFXGraphics::MeasureText(AEE_FONT_NORMAL, "Send");

    // Get the width and height of the screen
    SFXDevice device;
    UInt16 ScreenWidth = device.GetScreenWidth();
    SInt16 ScreenHeight = device.GetScreenHeight();

    // Calculate the position of buttons, in the center of the screen
    SInt16 position_X = (ScreenWidth - (width + 2 * MARGIN))/2;
    SInt16 position_Y = (ScreenHeight - (4 * fontHeight + 3 * MARGIN) - fontHeight)/2;
    SFXRectangle rectangle(position_X, position_Y, width + 2 * MARGIN, fontHeight + 2);

    // Create buttons
    SFRButtonControlPtr button1 = ::new SFRButtonControl(this, rectangle, "Send");

    rectangle.AddY(fontHeight + MARGIN);

    SFRButtonControlPtr button2 = ::new SFRButtonControl(this, rectangle, "Receive");

    rectangle.AddY(fontHeight + MARGIN);

    SFRButtonControlPtr button3 = ::new SFRButtonControl(this, rectangle, "Setup");

    rectangle.AddY(fontHeight + MARGIN);

    SFRButtonControlPtr button4 = ::new SFRButtonControl(this, rectangle, "End");

    // Button handlers registration
    if (static_try() && button1 != null) {
        static_throw(button1->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnButtonControl1)));
        if (static_try()) {
            button1->SetStatusFocus(true);
        }
    }
    if (static_try() && button2 != null) {
        static_throw(button2->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnButtonControl2)));
    }
    if (static_try() && button3 != null) {
        static_throw(button3->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnButtonControl3)));
    }
    if (static_try() && button4 != null) {
        static_throw(button4->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnButtonControl4)));
    }
    return;
}

//
// Destructor definition.
// Nothing special is done here.
//
MainWindow::~MainWindow(Void)
{
    return;
}

//
// Key handler (When the key is pressed and
// its corresponding window is in the foreground,
// it is executed).
//
HANDLER_IMPLEMENT_BOOLEVENT(MainWindow, OnKey, event)
{
    UInt16 key = event.GetP16();

    switch (key) {
        case AVK_UP:
            // Focus is moved up
            FocusUp();
            return true;
        case AVK_DOWN:
            // Focus is moved down
            FocusDown();
            return true;
        case AVK_CLR:
            // Application is ended
            SFRApplication::Terminate();
            return true;
    }
    return false;
}

//
// "Send" button press handler.
//
HANDLER_IMPLEMENT_VOIDCONTROL(MainWindow, OnButtonControl1, result, control)
{
    unused(result);
    unused(control);

    // The Send mail window is opened
    ::new SendingWindow();
}

//
// "Receive" button press handler.
//
HANDLER_IMPLEMENT_VOIDCONTROL(MainWindow, OnButtonControl2, result, control)
{
    unused(result);
    unused(control);

    // Prepare for and start receiving mail

    SFCError error;
    SFXConfigPtr option = SimpleMailer::GetOption();

    // Set account (user name and password)
    _receiver.SetAccount(option->ReadSFXAnsiString(OPTION_USER, ""), option->ReadSFXAnsiString(OPTION_PASSWORD, ""),
                        SFXPOP3Receiver::AUTH_ONLY_USERPASS);
        // SFXPOP3Receiver::AUTH_ONLY_USERPASS is the password authorization,
        // different with the mail servers
        // in this case it is the yahoo.co.jp mail server

   // Set POP3 server address and port number (the domain name is automatically resolved)
    _receiver.SetServer(SFXSocketAddress(option->ReadSFXAnsiString(OPTION_POP3SERVER, ""),
        option->ReadUInt16(OPTION_POP3PORT, 110)));

    // Register a callback for mail receiving
    // Test whether the received mail is deleted or not
    if (option->ReadBool(OPTION_ERASEMAIL, false)) {
        // Receive and delete
        error = _receiver.ReceiveAndDelete(CALLBACK_FUNCTION(POP3Callback));
    }
    else {
        // Receive mail
        error = _receiver.Receive(CALLBACK_FUNCTION(POP3Callback));
    }

    // If no error
    if (error == SFERR_NO_ERROR) {
        // Create info dialog
        _dialog = ::new SFRMessageDialog(SFXGraphics::GetDeviceRectangle()
            .Deflate(20, 100), "Receiving...", "");
    }
    else {
        // Create info dialog
        _dialog = ::new SFRMessageDialog(SFXGraphics::GetDeviceRectangle()
            .Deflate(20, 100), "Reception failed", "OK");
    }
}

//
// "Setup" button press handler.
//
HANDLER_IMPLEMENT_VOIDCONTROL(MainWindow, OnButtonControl3, result, control)
{
    unused(result);
    unused(control);

    // Create the options window
    ::new OptionWindow();
}

//
// "End" button press handler.
//
HANDLER_IMPLEMENT_VOIDCONTROL(MainWindow, OnButtonControl4, result, control)
{
    unused(result);
    unused(control);

    // Application is ended
    SFRApplication::Terminate();
}

//
// Callback executed when mail receiving is completed.
//
CALLBACK_IMPLEMENT_SFXPOP3RECEIVER(MainWindow, POP3Callback, error)
{
    SFXWideString message;
    SFXPropertyPtr mail;
    SInt32 i;

    if (_dialog != null) {
        // Close the "Receiving..." dialog
        _dialog->Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
    }
    if (error == SFERR_NO_ERROR) {
        message.Set("Successful reception.");
    }
    else {
        message.Set("Reception failed");
    }

    // Create a message dialog
    _dialog = ::new SFRMessageDialog(SFXGraphics::GetDeviceRectangle().Deflate(20, 100), message, "OK");

    // If no error
    if (error == SFERR_NO_ERROR) {
        if (_dialog != null) {
            // Dialog handler registration
            _dialog->RegisterHandler(SREVT_DIALOG, HANDLER_BEFORE, HANDLER_FUNCTION(OnDialog));
        }
        mail = SimpleMailer::GetMail();
        // The received mail is copied
        const SFXArray<SFXPOP3Receiver::MailInfoPtr>& mailArray = _receiver.GetReceivedMailArray();
        for (i = 0; i < mailArray.GetSize(); ++i) {
            SFXPOP3Receiver::MailInfoPtr minfo = mailArray[i];
            mail->Set(minfo->uidl, minfo->mail);
        }
    }

    // Redraw
    SFRApplication::GetInstance()->Invoke(SFXEvent(SREVT_RESPONDER_RENDER, SRP16_RENDER_INVOKE, false));
}

//
// Dialog handler.
//
HANDLER_IMPLEMENT_VOIDDIALOG(MainWindow, OnDialog, result, dialog)
{
    unused(result);

    // Dialog is closed
    dialog->Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
    // Create window
    ::new ReceivedWindow();

    // Redraw
    SFRApplication::GetInstance()->Invoke(SFXEvent(SREVT_RESPONDER_RENDER, SRP16_RENDER_INVOKE, false));
}
